/***************************************************************************
                          error.c  -  contains the error handling functions
                             -------------------
    begin                : Mon Apr 14 2003
    copyright            : (C) 2003 by red0x
    email                : red0x@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file plugins/error.c
  * File contains error handlers
  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "error.h"

#ifdef HAVE_STDLIB_H
#include	<stdlib.h>
#endif

#include 	<errno.h>
#include	<stdio.h>

#ifdef HAVE_STRING_H
#include	<string.h>
#endif

#include	<stdarg.h>      /* ANSI C header file */

#ifdef HAVE_SYSLOG_H
#include	<syslog.h>      /* for syslog() */
#endif

#include	"plugin.h"

static const char rcsid[] =
        "\100$ autosec/sonar: $Id: error.c,v 1.2 2003/09/28 12:05:33 red0x Exp $";

//extern int    daemon_proc;            /* set nonzero by daemon_init() */
extern int errno;
extern sonar_t mthis;

static void err_doit(int, int, const char *, va_list);


/* Nonfatal error related to a system call.
 * Print a message and return. */

void err_ret(const char *fmt, ...)
{
        va_list ap;

        va_start(ap, fmt);
        err_doit(1, LOG_INFO, fmt, ap);
        va_end(ap);
        return;
}

/* Fatal error related to a system call.
 * Print a message and terminate. */

void err_sys(const char *fmt, ...)
{
        va_list ap;

        va_start(ap, fmt);
        err_doit(1, LOG_ERR, fmt, ap);
        va_end(ap);
        exit(1);
}

/* Fatal error related to a system call.
 * Print a message, dump core, and terminate. */

void err_dump(const char *fmt, ...)
{
        va_list ap;

        va_start(ap, fmt);
        err_doit(1, LOG_ERR, fmt, ap);
        va_end(ap);
        abort();                /* dump core and terminate */
        exit(1);                /* shouldn't get here */
}

/* Nonfatal error unrelated to a system call.
 * Print a message and return. */

void err_msg(const char *fmt, ...)
{
        va_list ap;

        va_start(ap, fmt);
        err_doit(0, LOG_INFO, fmt, ap);
        va_end(ap);
        return;
}

/* Fatal error unrelated to a system call.
 * Print a message and terminate. */

void err_quit(const char *fmt, ...)
{
        va_list ap;

        va_start(ap, fmt);
        err_doit(0, LOG_ERR, fmt, ap);
        va_end(ap);
        exit(1);
}

/* Print a message and return to caller.
 * Caller specifies "errnoflag" and "level". */

static void err_doit(int errnoflag, int level, const char *fmt, va_list ap)
{
        int errno_save, n;
        char buf[MAXLINE];

        errno_save = errno;     /* value caller might want printed */
#ifdef	HAVE_VSNPRINTF
        vsnprintf(buf, sizeof(buf), fmt, ap);   /* this is safe */
#else
        vsprintf(buf, fmt, ap); /* this is not safe */
#endif
        n = strlen(buf);
        if(errnoflag)
                snprintf(buf + n, sizeof(buf) - n, ": %s",
                         strerror(errno_save));
        strcat(buf, "\n");

        if(mthis.sCon.bFork)
        {
                syslog(level, buf);
        }
        else
        {
                fflush(stdout); /* in case stdout and stderr are the same */
                fputs(buf, stderr);
                fflush(stderr);
        }
        return;
}
