/***************************************************************************
   inp_rand.c  -  random input plugin
                             -------------------
    begin                : Wed Jul 03 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: inp_rand.c,v 1.4 2003/11/07 08:41:07 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file inp_rand.c
  * Contains a random host input plugin
  */

#include "inp_rand.h"
#include <assert.h>

static const char rcsid[] =
        "\100$ autosec/sonar: $Id: inp_rand.c,v 1.4 2003/11/07 08:41:07 red0x Exp $";
	
/// Plugin's plugin_t structure, built by plugin_init
static plugin_t pthis;
/// Sonar's sonar_t structure, built by sonar, passed to this plugin
static sonar_t *mthis;

/// my last error
static int16_t my_errno;

static char my_hostname[25];
static unsigned int num_hosts;
	
static int plugin_destroy(void);
static int plugin_open(void *c_data, int i_data);
static int plugin_args(char opt, char *optarg);
static int plugin_close(void);
static int plugin_next(char *hostname);
static void plugin_usage(void);
static int plugin_last_error(void);
static void plugin_status(void);
static unsigned int get_random(void);
	
void *plugin_init(void *in_data)
{

	assert(in_data != NULL);

        //useless reference, to avoid a warning
        if(rcsid)
        {
        }
        // this is assumed
        mthis = (sonar_t *) in_data;
        if(mthis->api < 5)
	{
                fprintf(stderr, "Plugin API must be 5 or higher, update sonar or downgrade this plugin\n");
	}

        pthis.type = INPUT_PLUGIN;
        pthis.err = &my_errno;
        pthis.ready = false;
        pthis.selected = false;
	pthis.version = 5; /* set API version 5 */

        pthis.plugin_char = 'R';
        pthis.shortname = "random";
        pthis.longname = "Random Input Plugin";

        pthis.plugin_init = &plugin_init;
        pthis.plugin_destroy = &plugin_destroy;
        pthis.plugin_args = &plugin_args;
        pthis.plugin_open = &plugin_open;
        pthis.plugin_close = &plugin_close;
        pthis.plugin_next = &plugin_next;
        // this is not needed for net plugins
        pthis.plugin_sweep = NULL;
        pthis.plugin_output = NULL;
	pthis.plugin_results = NULL;
        pthis.plugin_usage = &plugin_usage;
        pthis.plugin_status = &plugin_status;
        pthis.plugin_last_error = &plugin_last_error;
        my_errno = PLUGIN_LOADED;
	
	srand((unsigned int ) time(NULL) ^ (unsigned int) &mthis);
	
	memset(my_hostname, 0, BUF_SIZE);
	/* default to one host */
	num_hosts = 1;

        return &pthis;
}	

static int plugin_destroy(void)
{

        mthis = NULL;

        pthis.type = 0;
        pthis.err = NULL;
        pthis.ready = false;
        pthis.plugin_init = NULL;
        pthis.plugin_destroy = NULL;
        pthis.plugin_open = NULL;
        pthis.plugin_close = NULL;
        pthis.plugin_next = NULL;
        pthis.plugin_sweep = NULL;
        pthis.plugin_args = NULL;
        pthis.plugin_sweep = NULL;
        pthis.plugin_output = NULL;
	pthis.plugin_results = NULL;
        pthis.plugin_usage = NULL;
        pthis.plugin_status = NULL;
        pthis.plugin_last_error = NULL;
        my_errno = PLUGIN_UNLOADED;
	
	memset(my_hostname, 0, 16);
        return my_errno;
}

static int plugin_open(void *c_data, int i_data)
{
	/* return the buffer size */
	return BUF_SIZE -1;
}

static int plugin_args(char opt, char *optarg)
{
        assert(mthis != NULL);
        switch (opt)
        {
		case 'n':
			assert(optarg != NULL);
			num_hosts = atoi(optarg);
			if(num_hosts < 0)
				num_hosts = 0;
			break;
                default:
                        //we dont handle this
                        return PLUGIN_ERROR;
        }
        return PLUGIN_OK;
}

static int plugin_close(void)
{
	memset(my_hostname, 0, BUF_SIZE);
        return PLUGIN_OK;
}

static int plugin_next(char *hostname)
{
	static unsigned int returned = 0;
	static char *myname;
	char myport[PORT_SIZE];
	unsigned long int addr, len;
	unsigned int port;
	struct in_addr ia;
	
	assert(hostname != NULL);
	
	if(returned >= num_hosts)
		return PLUGIN_ERROR;
	returned++;
	
	memset(myport, 0, PORT_SIZE);
	addr = (unsigned long int) get_random();
	port = get_random() % 65535;
	snprintf(myport, PORT_SIZE, "%u", port);
	
	ia.s_addr = addr;
	myname = inet_ntoa(ia);
	
	len = strlen(myname);
	memcpy(hostname, myname, len);
	strncat(hostname, ":", 1);
	strncat(hostname, myport, strlen(myport));
	if(mthis->sCon.iVerbose > 0)
		mthis->message("Scanning %s\n", hostname);
	
        return PLUGIN_OK;
}

static void plugin_status(void)
{
        
}

static unsigned int get_random(void)
{
	FILE *fp;
	unsigned int ret;
	if((fp = fopen("/dev/urandom", "r")) != NULL)
	{
		if((fread(&ret, sizeof(unsigned int), 1, fp)) == 1)
		{
			fclose(fp);
			return ret;
		}
	}
	if((fp = fopen("/dev/random", "r")) != NULL)
	{
		if(fread(&ret, sizeof(unsigned int), 1, fp) == 1)
		{
			fclose(fp);
			return ret;
		}
	}
	srand((unsigned int) clock());
	ret = 0x1008 + (unsigned int) (0xfffe * rand() / (RAND_MAX + 1.0));
	return ret;
}

static void plugin_usage(void)
{
	/* wont ever get called */
	
}

static int plugin_last_error(void)
{
	return my_errno;
}
