/***************************************************************************
   rfc793.h  -  TCP ACK Scan plugin
                             -------------------
    begin                : Wed Jul 03 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: rfc793.h,v 1.25 2003/10/01 07:13:29 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file rfc793.h
  * Contains a rfc 793 behavior check (TCP ACK->RST ping)
  */
#ifndef _RFC793_H_
#define _RFC793_H_

// include the plugin API
#include "plugin.h"

#ifdef MACOSX
#ifndef socklen_t
typedef int socklen_t;
#endif
#endif

#define LINUX 1
#define __FAVOR_BSD

#ifndef ICMP_DEST_UNREACH
#define ICMP_DEST_UNREACH       3       /* Destination Unreachable      */
#endif

#ifndef ICMP_PORT_UNREACH
#define ICMP_PORT_UNREACH       3       /* Port Unreachable             */
#endif

#ifndef SOL_IP
#define SOL_IP 0
#endif

#ifndef INADDR_NONE
#define INADDR_NONE	((in_addr_t) 0xffffffff)
#endif

#define MAX_PACKET 1024

#ifndef __FAVOR_BSD
#define __FAVOR_BSD
#endif
#ifndef __USE_BSD
#define __USE_BSD
#endif
#ifndef __BSD_SOURCE
#define __BSD_SOURCE
#endif

#define int_ntoa(x) inet_ntoa(*((struct in_addr *)&(x)))


// include my types
#include "types.h"
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/time.h>

/** 
 * Checksum stuff 
 */
#define CKSUM_CARRY(x) \
    (x = (x >> 16) + (x & 0xffff), (~(x + (x >> 16)) & 0xffff))

// put your definitions here
#define ETH_SIZE 14
#define IP_SIZE sizeof(struct ip)
#define TCP_SIZE sizeof(struct tcphdr)


/**
 * @struct _fakehead 
 * pseudo TCP header for calculating the chksum 
 */
struct _fakehead
{
        uint32_t saddr;
        uint32_t daddr;
        uint8_t zero;
        uint8_t protocol;
        uint16_t tot_len;
};

/** 
 * @struct net_tuple
 * leet net tuple 
 */
struct net_tuple
{
        uint32_t src;
        unsigned short int sport;
        uint32_t dst;
        unsigned short int dport;
};


struct recv_t
{
        /// socket to listen on (must already be open)
        int socket, errs;
        /// number of hosts we pinged
        uint32_t hosts;
        /// set the number of responses
        uint32_t responses;
        /// start time
        clock_t start;
        /// max wait time (timeout function)
        clock_t max;
};

// put your prototypes here
// make them static and make sure their linkage is "C"
void *plugin_init(void *in_data);
//static target_t *randomize(target_t * in);



#endif //_RFC793_H_
