/***************************************************************************
                          net.c  -  contains some network functions
                             -------------------
    begin                : Tues May 13 2003
    copyright            : (C) 2003 by red0x
    email                : red0x@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file net.c
  * File contains network handlers
  */
#include "net.h"
#include "error.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <strings.h>

static const char rcsid[] =
        "\100$ autosec/sonar: $Id: net.c,v 1.1 2003/05/14 06:23:03 red0x Exp $";

struct addrinfo *host_serv(const char *host, const char *serv, int family,
                           int socktype)
{
        int n;
        struct addrinfo hints, *res;

        bzero(&hints, sizeof(struct addrinfo));
        hints.ai_flags = AI_CANONNAME;  /* always return canonical name */
        hints.ai_family = family;       /* AF_UNSPEC, AF_INET, AF_INET6, etc. */
        hints.ai_socktype = socktype;   /* 0, SOCK_STREAM, SOCK_DGRAM, etc. */

        if((n = getaddrinfo(host, serv, &hints, &res)) != 0)
                return (NULL);

        return (res);           /* return pointer to first on linked list */
}

/* end host_serv */

/*
 * There is no easy way to pass back the integer return code from
 * getaddrinfo() in the function above, short of adding another argument
 * that is a pointer, so the easiest way to provide the wrapper function
 * is just to duplicate the simple function as we do here.
 */

struct addrinfo *Host_serv(const char *host, const char *serv, int family,
                           int socktype)
{
        int n;
        struct addrinfo hints, *res;

        bzero(&hints, sizeof(struct addrinfo));
        hints.ai_flags = AI_CANONNAME;  /* always return canonical name */
        hints.ai_family = family;       /* 0, AF_INET, AF_INET6, etc. */
        hints.ai_socktype = socktype;   /* 0, SOCK_STREAM, SOCK_DGRAM, etc. */

        if((n = getaddrinfo(host, serv, &hints, &res)) != 0)
                err_quit("host_serv error for %s, %s: %s",
                         (host == NULL) ? "(no hostname)" : host,
                         (serv == NULL) ? "(no service name)" : serv,
                         gai_strerror(n));

        return (res);           /* return pointer to first on linked list */
}
