/***************************************************************************
                          service.c  -  contains functions to resolve services
                             -------------------
    begin                : Sat Aug 30 2003
    copyright            : (C) 2003 by red0x
    email                : red0x@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file service.c
  * File contains functions to resolve services
  */



#include <netdb.h>
#include <assert.h>
#include "target.h"
#include "service.h"
#include "error.h"
#include "log.h"
#include "sonar.h"

static const char rcsid[] =
        "\100$ autosec/sonar: $Id: service.c,v 1.3 2003/10/01 08:18:55 red0x Exp $";

int sonar_resolve_service(char *serv, target_t *target)
{
	struct servent *service;
	int num = atoi(serv), port = 0;

log_write(LOG_DBUG, "  looking up \'%s\' service\n", serv);

        setservent(0);
        service = getservbyname(serv, NULL);
        if(!service)            // if not, 
        {
		log_write(LOG_DBUG, "  service not found by name: %s\n", serv);
                // try by number
                service = getservbyport(htons(num), NULL);
                if(!service)    // still not?
                {
			log_write(LOG_DBUG, "  service not found by port: %d\n", num);
                        // see if its in range
                        if(num >= 0 && num <= 65535)
			{
				log_write(LOG_DBUG, "  setting port #%d\n", num);
                                target->ports[num] = 1;
				port = num;
			}
                        else    // nope? error
                        {
                                err_msg("port out of range, skipping");
                                return 0;
                        }

                }
                else            // got it by number
                {
			log_write(LOG_DBUG, "  found service by number: %s\n", service->s_name);
			port = ntohs(service->s_port);
			if(port < 0 || port > 65535)
			{
				err_msg("port out of range, skipping");
				return 0;
			}   
                        target->ports[port] = 1;
                }
        }
        else                    // got it by name
        {
		log_write(LOG_DBUG, "  found service by name. port: %u, name: %s\n",
                        ntohs(service->s_port), service->s_name);
		port = ntohs(service->s_port);	
		if(port < 0 || port > 65535)
		{
			err_msg("port out of range, skipping");
			return 0;
		}   
                target->ports[port] = 1;
        }
        endservent();
	return port;
}


int sonar_resolve_service_r(char *low, char *high, target_t *result)
{
	int i, fin, moyen, ret = 0;
	assert(low != NULL);
	assert(high != NULL);
	moyen = sonar_resolve_service(low, result);
	fin = sonar_resolve_service(high, result);
	if(moyen < 0 || fin < 0)
	{
		err_msg("ports out of range, bailing");
		return 0;
	}

	// fill in the gaps in the range
	for(i = moyen; i < fin; i++)
	{
		result->ports[i] = 1;
		ret++;
	}
	ret++;
	return ret;
}
