/***************************************************************************
                          sonar.c  -  main sonar file
                             -------------------
    begin                : Mon Apr  8 17:26:46 PDT 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: sonar.c,v 1.26 2003/11/10 10:22:34 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file sonar.c
  * File contains most of the main functions of sonar
  */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <stdarg.h>
#include "types.h"
#include "sonar.h"
#include "util.h"
#include "plugin.h"
#include "target.h"
#include "net.h"
#include "popts.h"
#include "error.h"
#include "log.h"

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

static const char rcsid[] =
        "\100$ autosec/sonar: $Id: sonar.c,v 1.26 2003/11/10 10:22:34 red0x Exp $";

//these are contained in plugin.cpp
extern plugin_list_t input_list[];
extern plugin_list_t output_list[];
extern plugin_list_t network_list[];
extern int num_inp, num_out, num_net;
extern target_t *vectors;

poptContext optCon;             // context for parsing command-line options

sonar_t mthis;

void default_context(void *in)
{
	char *temp, *pv;
        sonar_t *mthis;

        assert(in != NULL);

        mthis = (sonar_t *) in;

        memset((void *) mthis, 0, sizeof(sonar_t));
	mthis->ver_major = 0;
        mthis->ver_minor = 0;
        mthis->revision = 0;
	pv = (char *) malloc(strlen(PACKAGE_VERSION));
	if(!pv)
		err_sys("malloc");
	strncpy(pv, PACKAGE_VERSION, strlen(PACKAGE_VERSION));
	temp = strpbrk(pv, ".");
	while(pv != NULL)
	{
		if(temp)
			*temp++ = 0x00;
		if(mthis->ver_major == 0)
			mthis->ver_major = atoi(pv);
		else if(mthis->ver_minor == 0)
			mthis->ver_minor = atoi(pv);
		else if(mthis->revision == 0)
			mthis->revision = atoi(pv);
		else
			pv = NULL;
		pv = temp;
		if(!pv)
			break;
		temp = strpbrk(pv, ".");
	}
	free(pv);
        // plugin API number
        mthis->api = API_VER;
        mthis->memput = &bput;
        mthis->message = &sonar_msg;
	mthis->resolve = &Host_serv;
        mthis->output = &sonar_rawoutput;
	mthis->results = &sonar_result_hook;
	mthis->insert_results = &insert_results;
	mthis->free_results = &free_results;
        mthis->usage = &sonar_usage;
        mthis->getprivs = &sonar_getprivs;
        mthis->dropprivs = &sonar_dropprivs;
	// no targets yet  ;)
	mthis->vectors = NULL;
	mthis->num_targets = 0;
	mthis->num_ports = 0;
}


/**
  * @fn main(int argc, char *argv[])
  * Main Method
  * @param argc Number of arguments
  * @param argv Arguments
  * @return SONAR_OK on success
  */
int main(int argc, char *argv[])
{

        int cur = 0, opt = 0;
        int plugin_selected = 0, i;
	target_t *first_target;
	struct timeval ts, te;
	float dif = 0;
        static char *plugin;

        struct poptOption optionsTable[] = {
                DEFAULT_OPTIONS {"help", '?', 0, 0, HELP, NULL, 0},
                {"usage", '\0', 0, 0, USAGE, NULL, 0},
                //POPT_AUTOHELP
                {NULL, 0, 0, NULL, 0}
        };


        //setup mthis (sonar_t)
        default_context(&mthis);

	mthis.my_name = argv[0];
        clear_sonar_context(&(mthis.sCon));
        clear_plugin_context(&(mthis.pCon));

        optCon = poptGetContext(NULL, argc, (const char **) argv,
                                optionsTable, 1);
        poptSetOtherOptionHelp(optCon, "[options] <scan-type> <target(s)>");

        //these will be fatal errors, as plugins become the 
        // core of sonar
        log_write(LOG_DBUG, "plugin_start(%p);\n", &mthis);

        if(plugin_start(&mthis) != PLUGIN_OK)
                sonar_popt_usage(optCon, -1, "Fatal",
                                 "Could not initialize plugin API\n");

        log_write(LOG_DBUG, "plugin_loadall(%p);\n", &mthis);

        if(plugin_loadall(&mthis) != PLUGIN_OK)
                sonar_popt_usage(optCon, -1, "Fatal",
                                 "Could not load any plugins\n");

        if(argc < 2)
        {
                sonar_popt_usage(optCon, 0, NULL, NULL);
        }

        while((opt = poptGetNextOpt(optCon)) >= 0)
        {
                // core options, handled first
                switch (opt)
                {

                        case PLUGIN_SELECT:
                                plugin = (char *) poptGetOptArg(optCon);
                                if(!plugin)
                                        sonar_popt_usage(optCon, 1, "fatal",
                                                         "no plugin selected");
                                for(i = 0; i < num_net; i++)
                                {
                                        int len, tlen = strlen(plugin);
                                        int len2 =
                                                strlen(network_list[i].plug->
                                                       shortname);
                                        int len3 =
                                                strlen(network_list[i].plug->
                                                       longname);
                                        len = (tlen < len2) ? tlen : len2;
                                        len2 = (tlen < len3) ? tlen : len3;
                                        if((strncmp
                                            (plugin,
                                             network_list[i].plug->shortname,
                                             len) == 0)
                                           ||
                                           (strncmp
                                            (plugin,
                                             network_list[i].plug->longname,
                                             len2) == 0)
                                           || (char) *plugin ==
                                           network_list[i].plug->plugin_char)
                                        {
                                                plugin_selected++;
                                                log_write(LOG_DBUG, "found network plugin \'%c\': %s\n", network_list[i].plug->plugin_char, network_list[i].plug->shortname);
                                                network_list[i].plug->
                                                        selected = true;
                                        }
                                }
                                break;
                        case INPUT_SELECT:
                                plugin = (char *) poptGetOptArg(optCon);
                                if(!plugin)
                                        sonar_popt_usage(optCon, 1, "fatal",
                                                         "no plugin selected");
                                for(i = 0; i < num_inp; i++)
                                {
                                        int len, tlen = strlen(plugin);
                                        int len2 =
                                                strlen(input_list[i].plug->
                                                       shortname);
                                        int len3 =
                                                strlen(input_list[i].plug->
                                                       longname);
                                        len = (tlen < len2) ? tlen : len2;
                                        len2 = (tlen < len3) ? tlen : len3;
                                        if((strncmp
                                            (plugin,
                                             input_list[i].plug->shortname,
                                             len) == 0)
                                           ||
                                           (strncmp
                                            (plugin,
                                             input_list[i].plug->longname,
                                             len2) == 0)
                                           || (char) *plugin ==
                                           input_list[i].plug->plugin_char)
                                        {
						log_write(LOG_DBUG, "found input plugin \'%c\': %s\n", input_list[i].plug->plugin_char, input_list[i].plug->shortname);
                                                plugin_selected++;
                                                input_list[i].plug->selected =
                                                        true;
                                        }
                                }
                                break;
                        case OUTPUT_SELECT:
                                plugin = (char *) poptGetOptArg(optCon);
                                if(!plugin)
                                        sonar_popt_usage(optCon, 1, "fatal",
                                                         "no plugin selected");
                                for(i = 0; i < num_out; i++)
                                {
                                        int len, tlen = strlen(plugin);
                                        int len2 =
                                                strlen(output_list[i].plug->
                                                       shortname);
                                        int len3 =
                                                strlen(output_list[i].plug->
                                                       longname);
                                        len = (tlen < len2) ? tlen : len2;
                                        len2 = (tlen < len3) ? tlen : len3;
                                        if((strncmp
                                            (plugin,
                                             output_list[i].plug->shortname,
                                             len) == 0)
                                           ||
                                           (strncmp
                                            (plugin,
                                             output_list[i].plug->longname,
                                             len2) == 0)
                                           || (char) *plugin ==
                                           output_list[i].plug->plugin_char)
                                        {
                                                plugin_selected++;
                                                log_write(LOG_DBUG, "found output plugin \'%c\': %s\n", output_list[i].plug->plugin_char, output_list[i].plug->shortname);
                                                output_list[i].plug->
                                                        selected = true;
                                        }

                                }
                                break;



                        case PLUGIN_OPTION:
                                plugin = (char *) poptGetOptArg(optCon);

                                log_write(LOG_DBUG, "plugin_command_handler(%c, %s);\n",
                                        *plugin, (char *) (plugin + 1));

                                if(plugin_command_handler
                                   (*plugin,
                                    (char *) (plugin + 1)) == SONAR_ERROR)
                                        sonar_usage
                                                ("Argument not handled by plugin(s)");
                                break;
                        case 'v':
                                mthis.sCon.iVerbose++;
                                log_write(LOG_DBUG, "verbosity = %d;\n",
                                        mthis.sCon.iVerbose);
                                break;
			case 'q':
				mthis.sCon.iVerbose = -1;
				break;
			case 'r':
				mthis.sCon.bRandom = 1;
				break;
		        case 'd':
				daemon_proc(argv[0], LOG_DAEMON);
				break;
                        case HELP:
                                sonar_popt_help(optCon, 1);
                                break;
                        case USAGE:
                                sonar_popt_usage(optCon, 1, NULL, NULL);
                                break;
                        default:
                                sonar_popt_usage(optCon, 1, "Bad Argument",
                                                 "unknown argument passed");
                                break;
                }
        }
	
        if(plugin_selected == 0)
        {
                sonar_popt_usage(optCon, 1, "fatal", "no plugin(s) selected");
        }
	
	/* no hostnames on the command line */
        if(poptPeekArg(optCon) == NULL)
        {
                int i, bypass = 0, iplug = 0, n;
		char *hostname;

                log_write(LOG_DBUG, "no hosts, checking for input plugs, then printing usage\n");

                for(i = 0; i < num_inp; i++)
                {
			/* see if we have input plugins selected */
                        if(input_list[i].plug->selected == true)
                        {
                                bypass++;
				iplug++;
                                assert(input_list[i].plug->plugin_open !=
                                      NULL);
				assert(input_list[i].plug->plugin_usage != NULL);
				/* input plugin not open */
                                if((n = input_list[i].plug->plugin_open(NULL, 0)) <= 0)
					input_list[i].plug->plugin_usage();
				
				/* n has the size of the hostname buffer we'll need */
				hostname = (char *) malloc(sizeof(char) * n);
				if(!hostname)
					err_sys("Could not malloc\n");
				do
				{
					memset(hostname, 0, n);
					if(input_list[i].plug->plugin_next(hostname) != PLUGIN_OK)
						break;
					sonar_add_target(&mthis, hostname);
				}while(1);
				free(hostname);
				break;
                        }
                }
		if(iplug == 0)
		{
			for(i = 0; i < num_out; i++)
			{
				if(output_list[i].plug->selected == true)
				{
					bypass++;
					assert(output_list[i].plug->plugin_usage !=
					       NULL);
					output_list[i].plug->plugin_usage();
				}
			}
			for(i = 0; i < num_net; i++)
			{
				if(network_list[i].plug->ready == true)
					bypass++;
				if(network_list[i].plug->selected == true)
				{
					bypass++;
					assert(network_list[i].plug->plugin_usage !=
					       NULL);
					network_list[i].plug->plugin_usage();
				}
			}
	
			if(bypass == 0)
			{
				sonar_popt_usage(optCon, -1, "ERROR",
						 "no target host/ip");
			}
		}
        }
	escape_shell(&(mthis.pCon.prog));
        sonar_commandline(argv, argc, &(mthis.sCon.sCommandLine));
        while(poptPeekArg(optCon) != NULL)
        {
                plugin = (char *) poptGetArg(optCon);
                // pass off anything left as targets

                log_write(LOG_DBUG, "sonar_add_target(%s);\n", plugin);
		sonar_add_target(&mthis, plugin);
	
        }

#ifdef DEBUG   
	sonar_debug_targets(mthis.vectors);
#endif
	if(mthis.sCon.bRandom)
		mthis.vectors = randomize(mthis.vectors);	/* randomize the targets */
	else	/* randomize takes care of looping, but if we didn't randomize, loop it */
	{
		first_target = mthis.vectors; 			/* all targets added, loop it */
		assert(mthis.vectors != NULL);
		while(mthis.vectors->next != NULL)
		{
			mthis.vectors = mthis.vectors->next;
		}
		mthis.vectors->next = first_target;
		mthis.vectors = first_target;
	}
	
        signal(SIGTERM, sig_int);
        signal(SIGINT, sig_int);
        signal(SIGHUP, sig_int);
#ifdef	DEBUG
	signal(SIGSEGV, sig_int);
#endif
	
        /* Status report here */
	if(mthis.sCon.iVerbose >= 0)
		sonar_status_report(mthis);
	
	/* hey, if we're broadcasting, we need time for the responses to come in */
	/* so, we could let the plugins handle it */
	/* timing */
	if(gettimeofday(&ts, NULL) != 0)
		err_ret("gettimeofday failed");
        for(i = 0; i < num_net; i++)
        {
                assert(network_list[i].plug != NULL);
		if(network_list[i].plug->selected)
			network_list[i].plug->plugin_open(NULL, 0);
        }
        while(mthis.sCon.iCount == -1 || cur < mthis.sCon.iCount)
        {
                int err;
                if((err = sonar_sweep(mthis.vectors)) == PLUGIN_ERROR)
                {
                        sonar_msg("Sweep ended with a plugin error\n");
                        return SONAR_ERROR;
                }
                if(err == PLUGIN_OK)    /* the meaning of this has changed slightly, */
                {			/* it just means everything went alright, no errors */

                }
                cur++;
                if((mthis.sCon.iCount > 1 || mthis.sCon.iCount == -1)
                   && (cur != mthis.sCon.iCount))
                        sleep(mthis.sCon.iScanDelay);
        }
	if(gettimeofday(&te, NULL) != 0)
		err_ret("gettimeofday failed");
	tvsub(&te, &ts);
	dif = te.tv_sec + te.tv_usec/1000000.0;
	/* XXX: i took LOG_SLOG out because it would hang in daemon mode */
	log_write(LOG_CONSOLE | LOG_PLUGINS, "Scan took %.2f seconds\n", dif);
	poptFreeContext(optCon);
        if(plugin_killall() != PLUGIN_OK)
                err_msg("Could not unload plugins\n");
        if(plugin_end() != PLUGIN_OK)
                err_msg("could not destroy plugins\n");
        

        return SONAR_OK;
}

/** @fn sonar_getprivs(void)
  * Get root priveleges when needed
  * @return uid of user before we got root
  */
int sonar_getprivs(void)
{
        uid_t uid;
        uid = getuid();
        log_write(LOG_DBUG, "  getuid() = %d;\n", uid);
        setreuid(-1, 0);
        log_write(LOG_DBUG, "  setreuid(-1,0);\n");

        if(geteuid())
                sonar_usage("must be root!");
        return uid;
}

/** @fn sonar_dropprivs(uid_t uid)
  * Drop root priveleges when no longer needed
  * Drops down to specified euid
  * @param uid UID_T of user to drop to
  * @return SONAR_OK 
  */
int sonar_dropprivs(uid_t uid)
{
        if(uid != geteuid())    /* no need to drop priveleges, its not setuid */
        {
                log_write(LOG_DBUG, "seteuid(%d);\n", uid);
                if(seteuid(uid) != 0)
                {
                        sonar_msg("could not drop priveleges, halting\n");
                        exit(-1);
                }
        }
        return SONAR_OK;
}

/** @fn sonar_rawoutput(char *msg,...)
  * Output formatted text given by msg and any other arguments.
  * Used in the Plugin API
  * @param msg Format string to output (see printf)
  * @return SONAR_OK 
  * @todo remove all dependency on this function.  Instead, switch to log_write
  */
int sonar_rawoutput(char *msg, ...)
{
        va_list ap;
	if(mthis.sCon.bFork)
                return SONAR_OK;
        
        va_start(ap, msg);
        vfprintf(mthis.sCon.oFile, msg, ap);
        va_end(ap);
        fflush(mthis.sCon.oFile);
        return SONAR_OK;
}

/** @fn sonar_status_report(sonar_t mthis)
  * Output a status report before each scan
  * @param mthis sonar_t structure
  */
void sonar_status_report(sonar_t mthis)
{
	char buf[128];
	struct tm *tm;
	time_t now = time(NULL);
	if (!(tm = localtime(&now))) 
		err_sys("Unable to get current localtime()#!#");
	
	/* ISO 8601 date/time: http://www.cl.cam.ac.uk/~mgk25/iso-time.html */ 
	if (strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M %Z", tm) <= 0)
		err_quit("Could not get date\n");
	if(!mthis.sCon.bFork && mthis.sCon.iVerbose >= 0)
	{
		log_write(LOG_CONSOLE | LOG_SLOG | LOG_PLUGINS, "starting sonar %s sweep at %s\n", VERSION, buf);
		if(mthis.sCon.iVerbose > 0)
			log_write(LOG_CONSOLE | LOG_DBUG | LOG_PLUGINS, "%s\n", mthis.sCon.sCommandLine);
		log_write(LOG_CONSOLE, "");
		plugin_status_hook();
		log_write(LOG_CONSOLE, "\n");
	}
}

/** @fn sonar_sweep(target_t *targets)
  * @param targets Pointer to list of targets
  * Run one pass of a sonar sweep
  * @return SONAR_ERROR if sweep failed
  * @return SONAR_OK if sweep worked
  * @todo Write some kind of error reporting/handling code, and have all the plugins return real error values
  */
int sonar_sweep(target_t *targets)
{
        //pick which network plugin to use, by getting the
        //ready one(s) and running their sweep functions.
        // these functions wont return until someone presses control-C, 
        // unless otherwise noted, at which point, program will return
        int status = PLUGIN_OK, plug;
        for(plug = 0; plug < num_net; plug++)
        {
                if(network_list[plug].plug == NULL)
                {
                        //broken plugin
                        err_msg("BROKEN PLUGIN DETECTED: id: %d -> %s",
                                plug, network_list[plug].plug->shortname);
                        continue;
                }
                {
                        assert(network_list[plug].plug != NULL);
                        assert(network_list[plug].plug->plugin_sweep != NULL);
                }

                // only run this sweep if the plugin is ready
                if(network_list[plug].plug->ready == true
                   && network_list[plug].plug->selected == true)
                {
                        status = network_list[plug].plug->plugin_sweep(targets);
#ifdef DEBUG
                        err_msg("network_list[%d].plug->plugin_sweep(%p); => %d", plug, targets, status);
#endif
			/* XXX: add some sort of error reporting here */
                }
                if(status != PLUGIN_OK)
                        return SONAR_ERROR;
        }
        return status;
}

/** @fn sonar_result_hook(result_t *results, int num_records)
  * @param results A linked list of results.
  * @param num_records How many results are in the list
  * @return SONAR_OK if all went well
  * @todo Fix up plugins to use this function to report results, instead of printing out
  */
int sonar_result_hook(result_t *results, unsigned long num_records)
{
	int i, ret = SONAR_OK;
	unsigned long j = 0; 
	assert(results != NULL);

	/* go through the number of records */
	/* we use '<=' here because we want a stub for hosts that did not return any info */
	while(j <= num_records && results != NULL)
	{
		/* go through the output plugins and send it to the selected ones */
		for(i=0; i < num_out; i++)
		{
			if(output_list[i].plug->selected && output_list[i].plug->ready)
			{
				/* let each plugin parse its own results */
				ret = output_list[i].plug->plugin_results(results);
				if(ret != PLUGIN_OK)
					log_write(LOG_CONSOLE | LOG_SLOG, "output plugin \'%s\' had trouble writing output\n", output_list[i].plug->shortname);
			}
		}
		j++;
		results = results->next;
	}
	
	return ret;
}
