/***************************************************************************
                          sonar.h  -  description
                             -------------------
    begin                : Tue May 15 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SONAR_H_
#define SONAR_H_

#include "plugin.h"
/// Error
#define SONAR_ERROR -1
/// Ok
#define SONAR_OK    1

#define DEFAULT_OPTIONS {"fork", 'f', POPT_ARG_NONE, 0, 'd', \
                 "fork into background (daemon mode)", NULL}, \
                {"verbose", 'v', POPT_ARG_NONE, 0, 'v', "increase verbosity", \
                 NULL}, \
		{"quiet", 'q', POPT_ARG_NONE, 0, 'q', "no output, just return", \
		NULL}, \
		{"randomize", 'r', POPT_ARG_NONE, 0, 'r', \
		"randomize target order", NULL} ,\
                {"scan_delay", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT, \
                 &(mthis.sCon.iScanDelay), 0, "delay between scans", \
                 "seconds"}, \
                {"scan_count", 'c', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT, \
                 &(mthis.sCon.iCount), 0, "number of times to scan", \
                 "cycles"}, \
                {"scan_type", 's', POPT_ARG_INT, 0, PLUGIN_SELECT, \
                 "scan plugin to use", "plugin"}, \
                {"input_plugin", 'i', POPT_ARG_INT, 0, INPUT_SELECT, \
                 "input plugin to use", "plugin"}, \
                {"output_plugin", 'o', POPT_ARG_INT, 0, OUTPUT_SELECT, \
                 "output plugin to use", "plugin"}, \
                {"plug_opt", 'p', POPT_ARG_STRING, 0, PLUGIN_OPTION, \
                 "plugin option", "option"}, \
                {"post_delay", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT, \
                 &(mthis.pCon.iPostDelay), 0, "delay after each scan type", \
                 "mseconds"}, \
                {"host_timeout", '\0', \
                 POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT, \
                 &(mthis.pCon.iTimeout), 0, "timeout for probes (per host)", \
                 "mseconds"}, \
		{"host_delay", '\0', \
		  POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT, \
		  &(mthis.pCon.iHostDelay), 0, "time to wait inbetween hosts", \
		"mseconds"}, \
                {"TTL", '\0', POPT_ARG_INT, &(mthis.pCon.iTTL), 0, \
                 "TTL for all scans applicable", "hops"}, \
                {"debug_sockets", '\0', POPT_ARGFLAG_DOC_HIDDEN, \
                 &(mthis.pCon.bSockDebug), 0, NULL, NULL}, \
                {"source_port", '\0', POPT_ARG_INT, &(mthis.pCon.iSrcPort), 0, \
                 "source port", "port"}, \
                {"packet_size", '\0', POPT_ARG_INT, &(mthis.pCon.iSize), 0, \
                 "packet size", "bytes"}, \
                {"pattern", '\0', POPT_ARG_STRING, &(mthis.pCon.sPattern), 0, \
                 "packet data pattern", "string"}, \
		{"exe", 'e', POPT_ARG_STRING, &(mthis.pCon.prog), 0, \
		"program to execute if host is online", "program"},

int h_errno;

int sonar_getprivs(void);
int sonar_dropprivs(uid_t);
void sonar_status_report(sonar_t mthis);
int sonar_rawoutput(char *msg, ...);
int sonar_result_hook(result_t *results, unsigned long num_records);
int sonar_sweep(target_t *targets);
void default_context(void *in);

/*! \mainpage sonar documentation and manual
 *
 * \section intro Introduction
 *
 * Sonar is used for network reconnaisance. Since release 1.0,
 * it has been plugin driven, with most of the control coming
 * from the plugins.  By default, version 1.0 comes with the
 * default ICMP sweep plugin.  This plugin can handle multiple
 * host names, so porting any scripts from sonar pre 1.0 to 1.0
 * should not be too much of a problem.  \n\n
 * Read INSTALL for generic instructions on compiling and installing.
 * This product is under the GNU GPL, which is available for reading
 * in COPYING.  Contributors, feel free to send me patches.  If you
 * include them in a release, you get your name and email (if you want)
 * in AUTHORS.  \n\n
 * This software comes with ABSOLUTELY NO WARRENTY, IMPLIED OR EXPRESSED!
 *
 * \section description Description
 *
 *  This software is used for the automatic probing of internet hosts
 *  at a timed interval (reconnaisance), checking host connectivity,  
 *  especially ICMP support (information gathering), automated running
 *  of third party tools when said host is online (automated testing).
 *  Using sonar is solely the user's or users' responsibility. In other words, 
 *  the author cannot be held responsible for your actions in any way.
 *
 *
 * \section cppapi Creating Plugins and the Plugin API
 * 
 *  See the \link pluginapi Plugin API Manual \endlink. The plugin API is 
 *  currently version 5.
 *
 *
 * \section install Installation
 *
 * \subsection step1 Step 1: Getting sources
 *  Sources are available at  http://autosec.sourceforge.net .  
 *  For those bleeding edgers out there,
 *  current work is available via anonymous CVS with the tag "HEAD". 
 *  Instructions are at http://sourceforge.net/cvs/?group_id=51027 .
 * \subsection step2 Step 2: Extraction and setup
 *  \c tar \c xzvf \c sonar-P-Q.tar.gz  \n
 *  \c cd \c sonar-P-Q  \n
 *  \c ./configure  \n\n
 *  Then edit config.h if you need to.
 * \subsection step3 Step 3: Compiling and Installing
 *  \c gmake  \n
 *  \c su \c root  \n
 *  \c gmake \c install  \n\n
 *  This will \b NOT install sonar SETUID root.  If you want sonar to be setuid root,
 *  type this:  \n\n
 *  \c chmod \c u+s \c /usr/local/bin/sonar \n\n
 * To change the install location, try this: \n
 * \c ./configure \c --prefix=/usr
 *  
 * \subsection help Further Help
 * Further help is available in the INSTALL and README files.
 *
 * \section probs Install Problems and Bugs
 *  If you are running a *nix platform, feel free to submit a 
 *  bug report about the compile process, or any other part of sonar.  
 *  Be sure to include \b ALL error messages during
 *  compile or crash, and feel free to attach core dumps, they help.  
 *  These reports should be submitted to 
 *  http://autosec.sourceforge.net/helpDesk/index.htm \n\n
 *  Known bugs are displayed at that site. 
 *
 * \section examples Syntax Examples
 *  \c sonar \c --TTL=250 \c -sI \c www.google.com  \n
 *    Sends pings to www.google.com with a TTL of 250 \n\n
 * \c sonar \c --scan_delay=5 \c -sI \c www.yahoo.com  \n
 *    Pings yahoo.com with a 5 second delay  \n\n
 * \c sonar \c -p \c p/etc/passwd \c --packet_size=300 \c -sI \c genbukan.no-ip.com  \n
 *    Sends the first 300 bytes of your passwd file to genbukan security
 *     (why would you want to do that?)  \n\n
 * \c sonar \c -f \c -oL \c -pf \c scan.log \c -c \c 5 \c --scan_delay=300 \c --TTL=220 \c -pe"echo \c google's \c up" \c -sI \c www.google.com  \n
 *    Puts sonar into the background (daemon mode), output to scan.log, send \n
 *     5 pings at most, once every 5 minutes, set the TTL to 220, and send  \n
 *     the pings to www.google.com. If the host responds, it prints "google's up"  \n
 *     on the command line.  \n\n
 * \c sonar  \n
 *    Gets the usage of sonar and all your plugins \n\n
 * \c sonar \c -sA \c www.google.com \n
 *    Does an ACK probe of google.com on all ports listed in /etc/services! \n
 *  Instead, try this:
 * \c sonar \c -sA \c www.google.com:80 \n
 *
 * \section contact Contact Information
 *   \c mail: \c red0x \c <red0x@users.sourceforge.net>  \n
 *   \c irc: \c irc.freenode.org \c - \c #autosec \c - \c red0x \n
 *   \c web: \c http://autosec.sourceforge.net/helpDesk/index.htm
 */



#endif
