/***************************************************************************
                          util.cpp  -  description
                             -------------------
    begin                : Wed Apr 17 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: util.c,v 1.21 2003/11/10 10:24:41 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file util.c
  * File contains some helper functions
  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "util.h"
#include "types.h"
#include "plugin.h"
#include "popts.h"
#include "error.h"
#include "log.h"
#include <pthread.h>
#include <assert.h>
#include <syslog.h>


extern int sonar_rawoutput(char *msg, ...);
extern int sonar_sweep(void);
extern poptContext optCon;
extern sonar_t mthis;

static const char rcsid[] =
        "\100$ autosec/sonar: $Id: util.c,v 1.21 2003/11/10 10:24:41 red0x Exp $";

/**
  * @fn sonar_usage(const char *err)
  * Print the usage with and error message
  * @param err Error to report
  */
void sonar_usage(const char *err)
{
        sonar_popt_usage(optCon, -1, "error", (char *) err);
}

/**
  * @fn bput(char ** dest, const char * src, const int len)
  * Copy memory from one place to another
  * @param dest Double pointer to destination
  * @param src Pointer the source
  * @param len Length of source
  * @todo Deprecate using this
  */
void bput(char **dest, const char *src, const int len)
{
        *dest = (char *) malloc(len + 2);
        memset(*dest, 0, len + 2);
        strncpy(*dest, src, len);
//  return dest;
}

/**
  * @fn sonar_commandline(char *argv[], int argc, char **out)
  * Get the commandline and store it in *out
  * @param argv argv from main
  * @param argc argc from main
  * @param out Double pointer to destination
  */
void sonar_commandline(char *argv[], int argc, char **out)
{
        int i, len = 0;

        for(i = 0; i < argc; i++)
                len += strlen(argv[i]) + 1;

        *out = (char *) malloc(len + 2);
        memset(*out, 0, len + 2);

        for(i = 0; i < argc; i++)
        {
                strncat(*out, argv[i], strlen(argv[i]));
                strcat(*out, " ");
        }
}

/**
  * @fn sonar_msg(char *msg, ...)
  * Print a message to plugins and syslog if in daemon mode, or to the plugins and console otherwise
  * @param msg Format specifier like printf
  */
void sonar_msg(char *msg, ...)
{
        va_list ap;
	char *buf;
		

	va_start(ap, msg);
	buf = (char *) malloc(128);
	if(!buf)
		err_sys("could not malloc");
	memset(buf, 0, 128);
	va_start(ap, msg);
	vsnprintf(buf, 127, msg, ap);
	
	if(mthis.sCon.bFork)
		log_write(LOG_PLUGINS | LOG_SLOG, buf);
	else
		log_write(LOG_PLUGINS | LOG_CONSOLE, buf);
	free(buf);
	va_end(ap);
}

RETSIGTYPE sig_int(int i)
{
	switch(i)
	{
	case SIGINT:
		err_msg("caught SIGINT, cleaning up..\n");
		break;
	case SIGTERM:
		err_msg("caught SIGTERM, cleaning up..\n");
		break;
	case SIGHUP:
		err_msg("caught SIGHUP, restarting..\n");
		sonar_sweep();	
		break;
	case SIGSEGV:
		err_msg("oops! sonar segmentation fault.\n");
		/*plugin_killall();*/
		abort();
		break;
	}
        /*plugin_killall();*/
	
        exit(0);
}

struct stat *get_stats(const char *file)
{
        struct stat *buf;
        char *path = NULL, *token, *str, cwd[PATH_MAX];

        assert(file != NULL);
        buf = (struct stat *) malloc(sizeof(struct stat));
        if(!buf)
                err_sys("get_stats: could not allocate space");


        log_write(LOG_DBUG, "   get_stats: checking for %s\n", file);

        // if we have the full path
        if(stat(file, buf) == 0)
                return buf;
        log_write(LOG_DBUG, "   get_stats: not found, checking in $PATH\n");

        path = getenv("PATH");
        // check in the paths
        if(path)
        {
                log_write(LOG_DBUG, "   get_stats: $PATH=\"%s\"\n", path);
                token = strtok(path, ":");
                while(token != NULL)
                {
                        // check this location for file
                        int tok = strlen(token);
                        int pth = strlen(file);
                        int len = tok + pth + 2;

                        str = (char *) malloc(len);
                        memset(str, 0, len);
                        // build the path name
                        strncpy(str, token, tok);
                        strncat(str, "/", 1);
                        strncat(str, file, pth);
			log_write(LOG_DBUG, "   get_stats: trying %s\n", str);
		
                        // try to stat
                        if(stat(str, buf) == 0)
                                return buf;
                        // not found, clear the mem
                        memset(str, 0, len);
                        // free the mem
                        free(str);
                        // next token
                        token = strtok(NULL, ":");
                }
        }
        // not found in path? check current working dir
        memset(cwd, 0, PATH_MAX);
        getcwd(cwd, PATH_MAX - 1);
	log_write(LOG_DBUG, "   get_stats: trying cwd\n");
        // we can get the cwd
        if(cwd)
        {
                // check here for file
                int pth = strlen(file);
                int lcw = strlen(cwd);
                int len = pth + lcw + 2;

                str = (char *) malloc(len);
                memset(str, 0, len);
                // build the path name
                strncpy(str, cwd, lcw);
                strncat(str, "/", 1);
                strncat(str, file, pth);
		log_write(LOG_DBUG, "   get_stats: trying %s\n", str);
                // try to stat
                if(stat(str, buf) == 0)
                        return buf;
                // not found, clear mem
                memset(str, 0, len);
                // free
                free(str);
        }
	log_write(LOG_DBUG, "   could not find/stat %s\n", file);
        return NULL;
}

RETSIGTYPE sig_chld(int sig)
{
	pid_t pid;
	int stat;
	
	pid = wait(&stat);
#if RETSIGTYPE == void
	return;
#else
	return (0);
#endif
}

/**
  * @fn daemon_proc(const char *pname, int facility)
  * Set things up for a daemon process 
  * @param pname Program name to report in syslog
  * @param facility Facility under which to open syslog
  * @todo This function closes file descriptors associated with output plugins, We need to fix that
  */
void daemon_proc(const char *pname, int facility)
{
        int i;
        pid_t pid;

	assert(pname != NULL);
	log_write(LOG_DBUG, "  Going into daemon mode\n");

        if((pid = fork()) != 0)
                exit(0);        /* parent terminates */

        /* 1st child continues */
        setsid();               /* become session leader */

        signal(SIGHUP, SIG_IGN);
	signal(SIGCHLD, sig_chld);
        if((pid = fork()) != 0)
                exit(0);        /* 1st child terminates */

        /* 2nd child continues */
        mthis.sCon.bFork = 1;        /* for our err_XXX() functions */

        chdir("/");             /* change working directory */

        umask(0);               /* clear our file mode creation mask */

	/* XXX: changed MAXFD to 3 */
        for(i = 1; i < MAXFD; i++)
                close(i);

        openlog(pname, LOG_PID, facility);
}

/** 
  * @fn escape_shell(char **myfunc)
  * Escape shell meta-characters from strign myfunc
  * @param myfunc String with possible meta-characters to be escaped
  * @return escaped string in the pointer
  */
void escape_shell(char **myfunc)
{
	char *func;
        int len;
        int i = 0;
	if(myfunc == NULL)
		return;
	if(*myfunc == NULL)
		return;
	
	func = *myfunc;
	len = strlen(func);
        for(i = 0; i < len; i++)
        {
                if(func[i] == '>' || func[i] == '<' ||
                   func[i] == ';' || func[i] == '(' ||
                   func[i] == '`' || func[i] == '!' ||
                   func[i] == '|' || func[i] == '&' ||
                   func[i] == '*' || func[i] == '?' ||
                   func[i] == '$' || func[i] == ':' ||
                   func[i] == '=' || func[i] == '#' ||
                   func[i] == '!' || func[i] == '%' ||
                   func[i] == ')' || func[i] == '[' ||
                   func[i] == ']' || func[i] == '{' || func[i] == '}' ||
                   /* somone trying to pass shellcode w/nop's? */
                   func[i] == (char) 0x90)
                        //we want to get rid of these kinds of characters
                        //so, move the string forward
                {
			
                        char *tmp;
                        if(i > 0)
                                if(func[i - 1] == '\\')
                                        continue;
			log_write(LOG_DBUG, "   :Found Meta-char(\\%c): escaping\n", func[i]);

                        // now we will escape the offending char
			// make temp storage, one larger than offending string
                        tmp = (char *) malloc(++len);
			if(!tmp)
				err_sys("Could not malloc\n");
                        // clear it
                        memset(tmp, 0, len);
                        // move the first half of the string to the temp
                        memmove(tmp, func, i);
                        // inject the escape char
                        tmp[i] = '\\';
                        // move second half of string to the temp
                        memmove(&tmp[i + 1], &func[i], len - (i + 1));
                        // swap temp and active variables
                        realloc(func, len + 1);
                        memset(func, 0, len + 1);
                        memmove(func, tmp, len);
                        free(tmp);

                }
        }
}

/**
  * @fn tvsub(register struct timeval *out, register struct timeval *in)
  * Subtract one struct timeval from another
  * @param out Pointer to destination (will be filled with out - in)
  * @param in Pointer to src
*/
void tvsub(register struct timeval *out, register struct timeval *in)
{
        if((out->tv_usec -= in->tv_usec) < 0)
        {
                out->tv_sec--;
                out->tv_usec += 1000000;
        }
        out->tv_sec -= in->tv_sec;
}

/**
  * @fn randomize(target_t * in)
  * Randomize the list of targets
  * @param in Final, filled in linked list of results, before you make it circular
  * @return A copy of the linked list in random order
  */
target_t *randomize(target_t * in)
{
        target_t *hosts, *hosts2;
        int i;
        unsigned int rand_int, num_hosts = mthis.num_targets;

	assert(in != NULL);
	
        /* if we dont have enough to change or we dont want it random */
	if(!in->next || mthis.num_targets == 1 || !mthis.sCon.bRandom)
                return in;

        /* get me a couple of  arrays */
        hosts = (target_t *) malloc(sizeof(target_t) * num_hosts);
        if(!hosts)
                return in;
        hosts2 = (target_t *) malloc(sizeof(target_t) * num_hosts);
        if(!hosts2)
                return in;
        memset(hosts, 0, sizeof(target_t) * num_hosts);
        memset(hosts2, 0, sizeof(target_t) * num_hosts);

        if(in->addrinfo == 0)
                in = in->next;

        /* get the source array */
        for(i = 0; i < num_hosts; i++)
        {
                if(in != NULL)
                {
			hosts[i].entry = in->entry;
                        memcpy(hosts[i].ports, in->ports, 65537 * sizeof(int));
                        hosts[i].addrinfo = in->addrinfo;
			hosts[i].netmask = in->netmask;
			hosts[i].extra_hosts = in->extra_hosts;
			hosts[i].num_ports = in->num_ports;
                        hosts[i].next = in->next;
                        in = in->next;
                }
        }

        /* build the new array */
        for(i = 0; i < num_hosts; i++)
        {
                /* get a random record */
                rand_int =
                        (unsigned int) (256.0 * rand() / (RAND_MAX + 0.0)) %
                        num_hosts;
              test:
                if(hosts[rand_int].addrinfo == 0)
                {
                        rand_int++;
                        rand_int %= num_hosts;
                        goto test;
                }

                hosts2[i].addrinfo = hosts[rand_int].addrinfo;
		memcpy(hosts2[i].ports, hosts[rand_int].ports, 65537 * sizeof(int));
		hosts2[i].netmask = hosts[rand_int].netmask;
		hosts2[i].num_ports = hosts[rand_int].num_ports;
		hosts2[i].extra_hosts = hosts[rand_int].extra_hosts;
                hosts2[i].entry =
                        (char *) malloc(strlen(hosts[rand_int].entry) + 2);
                memset(hosts2[i].entry, 0, strlen(hosts[rand_int].entry) + 2);
                if(!hosts2[i].entry)
                {
                        err_msg("ERROR MOVING HOSTNAME: randomizing aborted\n");
                        return in;
                }
                strncpy(hosts2[i].entry, hosts[rand_int].entry,
                        strlen(hosts[rand_int].entry));
		/* XXX: 
		free(hosts[rand_int].entry);*/

                /* link it */
                if(i > 0)
                        hosts2[i - 1].next = &hosts2[i];

                memset(&hosts[rand_int], 0, sizeof(target_t));
                /*printf("Profile: hosts2[%d] = %s\n", i, hosts2[i].entry);
		printf(" hosts2[%d] = hosts[%d]\n", i, rand_int);*/
        }
        // make it circular
	hosts2[num_hosts - 1].next = &hosts2[0];

        //remove the old one
        assert(hosts != NULL);
        free(hosts);
        //return the pointer
        return hosts2;
}

static int copy_ainfo(struct addrinfo **dest, struct addrinfo *src, int num)
{
	int len;
	struct addrinfo *first = NULL;

	/* for some reason, we didn't want to copy anything */
	if(num < 1)
		return PLUGIN_OK;
	
	if(!(*dest))
	{
		(*dest) = (struct addrinfo *) malloc(sizeof(struct addrinfo));
		if(!(*dest))
			err_sys("memory exhausted\n");
		memset((*dest), 0, sizeof(struct addrinfo));
	}
	
	first = *dest;
	while(num > 0)
	{
		(*dest)->ai_flags = src->ai_flags;
		(*dest)->ai_family = src->ai_family;
		(*dest)->ai_socktype = src->ai_socktype;
		(*dest)->ai_protocol = src->ai_protocol;
		(*dest)->ai_addrlen = src->ai_addrlen;
		(*dest)->ai_addr = (struct sockaddr *) malloc(src->ai_addrlen);
		if(!(*dest)->ai_addr)
			err_sys("memory exhausted\n");
		memset((*dest)->ai_addr, 0, src->ai_addrlen);
		memcpy((*dest)->ai_addr, src->ai_addr, src->ai_addrlen);
		if(src->ai_canonname)
		{
			len = strlen(src->ai_canonname);
			(*dest)->ai_canonname = (char *) malloc(len+1);
			if(!(*dest)->ai_canonname)
				err_sys("Memory exhausted\n");
			memset((*dest)->ai_canonname, 0, len+1);
			strncpy((*dest)->ai_canonname, src->ai_canonname, len);
		}
		else
		{
			(*dest)->ai_canonname = NULL;
		}
		(*dest)->ai_next = NULL;
		if(--num > 0) /* we need more of the list */
		{
			/* this only makes sense if there is more to copy */
			assert(src->ai_next != NULL);
			(*dest)->ai_next = (struct addrinfo *) malloc(sizeof(struct addrinfo));
			if(!(*dest)->ai_next)
				err_sys("memory exhausted\n");
			/* advance the lists */
			(*dest) = (*dest)->ai_next;
			src = src->ai_next;
		}
	}
	*dest = first;
	return PLUGIN_OK;
}

/**
  * @fn insert_results(result_t *results, result_t *insertion)
  * Insert a result into the results linked list
  * @param results Results linked list
  * @param insertion result_t structure to insert
  * @return PLUGIN_OK if everything went well.
  */
int insert_results(result_t *results, result_t *insertion)
{
	int len, i, match = 0;
	size_t in_len, cmp_len;
	result_t *wait = NULL;
	assert(results != NULL);
	assert(insertion != NULL);
	assert(insertion->ainfo != NULL);
	assert(insertion->ainfo->ai_addr != NULL);
	
	/* we are comparing by address data */
	in_len = insertion->ainfo->ai_addrlen;
	while(results)
	{
		/* no records yet */
		if(!(results->ainfo))
		{
			/* copy the address info */
			copy_ainfo(&(results->ainfo), insertion->ainfo, 1);
			/* straight up copy it, no need to remalloc */
			/*memcpy(results, insertion, sizeof(result_t));  XXX: doing this memcpy will 
			overwrite the addrinfo, causing random segfaults later*/
			results->hostname = insertion->hostname;
			memcpy(results->ports, insertion->ports, sizeof(unsigned char) * 65537);
			results->rtt = insertion->rtt;
			results->ttl = insertion->ttl;
			results->user_data = insertion->user_data;
			results->next = NULL;
			/* we're done, dood */
			return PLUGIN_OK;
		}
		/* this crap should not happen */
		assert(results->ainfo != NULL);
		assert(results->ainfo->ai_addr != NULL);
		
		/* do we have a match? */
		cmp_len = results->ainfo->ai_addrlen;
		len = (in_len < cmp_len) ? in_len: cmp_len;
		if(memcmp(results->ainfo->ai_addr, insertion->ainfo->ai_addr, len) == 0)
		{
			match++;
			/* the only thing that should be different is the ports, so update that */
			for(i=0; i < 65537; i++)
			{
				/* port has a state we wish to update associated with it */
				if(results->ports[i] < PORT_OPENED)
					results->ports[i] = insertion->ports[i];
			}	
		}
		
		wait = results;
		results = results->next;
	}
	/* no addresses matched, so make a new one */
	if(!match)
	{
		int len;
		/* we need a new record */
		/* rewind the list*/
		results = wait;
		/* insert a new record */
		results->next = (result_t *) malloc(sizeof(result_t));
		if(!(results->next))
			err_sys("No memory\n");
		/* fast forward */
		results = results->next;
		memset(results, 0, sizeof(result_t));
		/* remalloc the addrinfo so as not to get cleared out */
		copy_ainfo(&(results->ainfo), insertion->ainfo, 1);
		results->hostname = insertion->hostname;
		memcpy(results->ports, insertion->ports, sizeof(unsigned char) * 65537);
		results->rtt = insertion->rtt;
		results->ttl = insertion->ttl;
		results->user_data = insertion->user_data;
		len = strlen(insertion->hostname);
		/*results->ainfo->ai_canonname = (char *) malloc(len +1);
		if(!results->ainfo->ai_canonname)
			err_sys("Memory exhausted\n");
		memset(results->ainfo->ai_canonname, 0, len + 1);
		strncpy(results->ainfo->ai_canonname, insertion->hostname, len);*/
		results->next = NULL;
	}
	return PLUGIN_OK;
}

#define xfree(x) free(x); x = NULL

/** 
  * @fn free_results(result_t *results)
  * Free the memory associated with a results linked list
  * @param results Results linked list to free.
  */
void free_results(result_t *results)
{
	result_t *wait;
	
	while(results != NULL)
	{
		wait = results;
		
		/* XXX: sometimes this dies */
		freeaddrinfo(results->ainfo);
		
		xfree(results->ainfo);
		xfree(results->user_data);
		xfree(results->hostname);
		results = results->next;
		/* for some reason, this dies */
		/*xfree(wait);*/
	}
}
